﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace TPLSamples.Models
{
    public class CancellationSample : Sample
    {
        public override string SampleName
        {
            get { return "Cancellation Sample"; }
        }

        public override bool ImageRequired
        {
            get { return false; }
        }

        bool isRunning = false;
        public bool IsRunning
        {
            get { return this.isRunning; }
            set
            {
                if (this.isRunning != value)
                {
                    this.isRunning = value;
                    this.RaisePropertyChanged("IsRunning");
                }
            }
        }
                
        public CancellationTokenSource CancellationTokenSource { get; set; }

        private static Random ran = new Random();

        public override void Run(System.Drawing.Bitmap bmp = null, Action<string> UpdateLog = null)
        {
            Stopwatch s = new Stopwatch();
            s.Start();

            IsRunning = true;

            CancellationTokenSource = new CancellationTokenSource();
            CancellationTokenSource.Token.Register(() => { IsRunning = false; });

            var options = new ParallelOptions { CancellationToken = CancellationTokenSource.Token };
            try
            {
                Parallel.ForEach(WhileTrue(), options, i =>
                {
                    while (!options.CancellationToken.IsCancellationRequested)
                    {
                        UpdateLog("Sleeping in Cancellation Sample at " + i);
                        Thread.Sleep(ran.Next(1000));
                    }
                });
            }
            catch (OperationCanceledException)
            {
                UpdateLog("Operation has been cancelled."); ;
            }

            s.Stop();
            RunTime = s.Elapsed;
        }

        public static IEnumerable<int> WhileTrue()
        {
            for (int i = 0; ; i++)
                yield return i;
        }
    }
}
