﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Controls;
using System.Windows.Shapes;
using TPLSamples.Utility;

namespace TPLSamples.Models
{
	public class LineSample : Sample
	{
		public override string SampleName
		{
			get { return "Line Sample"; }
		}

		public override bool ImageRequired
		{
			get { return true; }
		}

		public override void Run(System.Drawing.Bitmap bmp = null, Action<string> UpdateLog = null)
		{
			if(bmp == null)
				throw new InvalidOperationException("Bitmap must be defined.");
			
			double X1 = 0, Y1 = 0;
			double X2 = bmp.Width - 1, Y2 = bmp.Height - 1;

			Stopwatch s = new Stopwatch();
			s.Start();

			double slope = (Y2 - Y1) / (X2 - X1);
			double beta = Y1 - (slope * X1);

			System.Drawing.Imaging.BitmapData bmData = bmp.LockBits(new System.Drawing.Rectangle(0, 0, bmp.Width, bmp.Height), System.Drawing.Imaging.ImageLockMode.ReadWrite, System.Drawing.Imaging.PixelFormat.Format24bppRgb);
			int stride = bmData.Stride;
			System.IntPtr Scan0 = bmData.Scan0;
			unsafe
			{
				byte* startPos = (byte*)(void*)Scan0;
				for (int x = (int)X1; x <= X2; x++)
				{
					GeneralMathOperations.DrawPixelByPointSlope(slope, beta, stride, startPos, x);
				}
			}
			bmp.UnlockBits(bmData);

			s.Stop();
			RunTime = s.Elapsed;
		}
	}
}
